<?php
// $Revision: 1.1.2.2 $
//  ------------------------------------------------------------------------ //
//  XooNIps - Neuroinformatics Base Platform System                          //
//  Copyright (C) 2005-2007 RIKEN, Japan All rights reserved.                //
//  http://xoonips.sourceforge.jp/                                           //
//  ------------------------------------------------------------------------ //
//  This program is free software; you can redistribute it and/or modify     //
//  it under the terms of the GNU General Public License as published by     //
//  the Free Software Foundation; either version 2 of the License, or        //
//  (at your option) any later version.                                      //
//                                                                           //
//  You may not change or alter any portion of this comment or credits       //
//  of supporting developers from this source code or any supporting         //
//  source code which is considered copyrighted (c) material of the          //
//  original comment or credit authors.                                      //
//                                                                           //
//  This program is distributed in the hope that it will be useful,          //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of           //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            //
//  GNU General Public License for more details.                             //
//                                                                           //
//  You should have received a copy of the GNU General Public License        //
//  along with this program; if not, write to the Free Software              //
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA //
//  ------------------------------------------------------------------------ //

if ( ! defined( 'XOOPS_ROOT_PATH' ) ) exit();

require_once dirname( __FILE__ ).'/unicode.class.php';

define( 'XOONIPS_FEED_DATE_RDF', 'Y-m-d\TH:i:s+00:00' );
define( 'XOONIPS_FEED_DATE_RSS', 'D, d M Y H:i:s T' );
define( 'XOONIPS_FEED_DATE_ATOM', 'Y-m-d\TH:i:s\Z' );

/**
 * The XooNIps Feed (RDF,RSS,Atom) generation class
 *
 * @package xoonips_base
 * @copyright copyright &copy; 2005-2007 RIKEN Japan
 * @author  Yoshihiro OKUMURA <orrisroot@users.sourceforge.jp>
 */
class XooNIpsFeed {
  var $unicode;
  var $id;
  var $site_title;
  var $site_url;
  var $site_language;
  var $site_author;
  var $site_description;
  var $site_copyright;
  var $items;

  /**
   * constructor
   * @access public
   * @param string $feed_url url for feeder
   */
  function XooNIpsFeed( $feed_url ) {
    global $xoopsConfig;
    $config_handler =& xoops_gethandler( 'config' );
    if ( defined( 'XOOPS_CUBE_LEGACY' ) ) {
      // for Cube 2.1
      $xoopsConfigMetaFooter =& $config_handler->getConfigsByDirname( 'legacyRender' );
    } else {
      // for Cube 2.0
      $xoopsConfigMetaFooter =& $config_handler->getConfigsByCat( XOOPS_CONF_METAFOOTER );
    }
    $this->unicode =& XooNIpsUnicode::getInstance();
    $this->feed_url = $feed_url;
    $this->site_title = htmlspecialchars( $this->unicode->encode_utf8( $xoopsConfig['sitename'] ), ENT_QUOTES, 'UTF-8' );
    $this->site_url = XOOPS_URL.'/';
    $this->site_language = _LANGCODE;
    $this->site_author = htmlspecialchars( $this->unicode->encode_utf8( $xoopsConfigMetaFooter['meta_author'] ), ENT_QUOTES, 'UTF-8' );
    $this->site_description = htmlspecialchars( $this->unicode->encode_utf8( $xoopsConfigMetaFooter['meta_description'] ), ENT_QUOTES, 'UTF-8' );
    $this->site_copyright = htmlspecialchars( $this->unicode->encode_utf8( $xoopsConfigMetaFooter['meta_copyright'] ), ENT_QUOTES, 'UTF-8' );
    $this->site_copyright = preg_replace( '/&amp;copy;/', '&#169;', $this->site_copyright );
    $this->site_copyright = preg_replace( '/&amp;#/', '&#', $this->site_copyright );
    $this->items = array();
  }

  /**
   * add item
   *
   * @access public
   * @param string $category category
   * @param string $title title
   * @param string $description description
   * @param string $link link url
   * @param int $timestamp timestamp
   */
  function addItem( $category, $title, $description, $link, $timestamp ) {
    $this->items[] = array( 
      'category' => htmlspecialchars( $this->unicode->encode_utf8( $category ), ENT_QUOTES, 'UTF-8' ),
      'title' => htmlspecialchars( $this->unicode->encode_utf8( $title ), ENT_QUOTES, 'UTF-8' ),
      'description' => htmlspecialchars( $this->unicode->encode_utf8( $description ), ENT_QUOTES, 'UTF-8' ),
      'link' => $link,
      'timestamp' => $timestamp,
    );
  }

  /**
   * render feeds
   *
   * @access public
   * @param string $type feed type belows:
   *   'rdf'  : RSS 1.0
   *   'rss'  : RSS 2.0
   *   'atom' : Atom 1.0
   */
  function render( $type ) {
    // sort by timestamp
    usort( $this->items, array( 'XooNIpsFeed', '_compare_items' ) );
    switch ( $type ) {
    case 'rdf':
      $this->_renderRDF();
      break;
    case 'rss':
      $this->_renderRSS();
      break;
    case 'atom':
      $this->_renderATOM();
      break;
    }
    exit();
  }

  /**
   * compare items for item sorting
   *
   * @access private
   * @param array $a item A
   * @param array $b item B
   * @return int result
   */
  function _compare_items( $a, $b ) {
    if ( $a['timestamp'] == $b['timestamp'] ) {
      return 0;
    }
    return ( $a['timestamp'] < $b['timestamp'] ) ? 1 : -1;
  }

  /**
   * output RDF feeds (RSS 1.0 : RDF Site Summary)
   *
   * @access private
   */
  function _renderRDF() {
    // header
    header( 'Content-type: application/rss+xml' );
    echo '<?xml version="1.0" encoding="UTF-8" ?>'."\n";
    echo '<rdf:RDF';
    echo ' xmlns="http://purl.org/rss/1.0/"';
    echo ' xmlns:dc="http://purl.org/dc/elements/1.1/"';
    echo ' xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"';
    echo '>'."\n";
    echo '<channel rdf:about="'.$this->site_url.'">'."\n";
    echo '<title>'.$this->site_title.'</title>'."\n";
    echo '<link>'.$this->site_url.'</link>'."\n";
    echo '<description>'.$this->site_description.'</description>'."\n";
    echo '<dc:language>'.$this->site_language.'</dc:language>'."\n";
    echo '<dc:rights>'.$this->site_copyright.'</dc:rights>'."\n";
    echo '<dc:date>'.gmdate( XOONIPS_FEED_DATE_RDF, time() ).'</dc:date>'."\n";
    echo '<dc:publisher>'.$this->site_author.'</dc:publisher>'."\n";
    echo '<dc:creator>'.$this->site_author.'</dc:creator>'."\n";
    echo '<items>'."\n";
    echo '<rdf:Seq>'."\n";
    foreach ( $this->items as $item ) {
      echo '<rdf:li rdf:resource="'.$item['link'].'"/>'."\n";
    }
    echo '</rdf:Seq>'."\n";
    echo '</items>'."\n";
    echo '</channel>'."\n";
    // entries
    foreach ( $this->items as $item ) {
      $date = gmdate( XOONIPS_FEED_DATE_RDF, $item['timestamp'] );
      echo '<item rdf:about="'.$item['link'].'">'."\n";
      echo '<title>'.$item['title'].'</title>'."\n";
      echo '<link>'.$item['link'].'</link>'."\n";
      echo '<description>'.$item['description'].'</description>'."\n";
      echo '<dc:subject>'.$item['category'].'</dc:subject>'."\n";
      echo '<dc:date>'.$date.'</dc:date>'."\n";
      echo '</item>'."\n";
    }
    // footer
    echo '</rdf:RDF>'."\n";
  }

  /**
   * output RSS feeds (RSS 2.0 : Really Simple Summary)
   *
   * @access private
   */
  function _renderRSS() {
    // header
    header( 'Content-type: application/rss+xml' );
    echo '<?xml version="1.0" encoding="UTF-8" ?>'."\n";
    echo '<rss version="2.0">'."\n";
    echo '<channel>'."\n";
    echo '<title>'.$this->site_title.'</title>'."\n";
    echo '<link>'.$this->site_url.'</link>'."\n";
    echo '<description>'.$this->site_description.'</description>'."\n";
    echo '<language>'.$this->site_language.'</language>'."\n";
    // entries
    foreach ( $this->items as $item ) {
      $date = gmdate( XOONIPS_FEED_DATE_RSS, $item['timestamp'] );
      echo '<item>'."\n";
      echo '<title>'.$item['title'].'</title>'."\n";
      echo '<link>'.$item['link'].'</link>'."\n";
      echo '<pubDate>'.$date.'</pubDate>'."\n";
      echo '<guid>'.$item['link'].'</guid>'."\n";
      echo '<description>'.$item['description'].'</description>'."\n";
      echo '<category>'.$item['category'].'</category>'."\n";
      echo '</item>'."\n";
    }
    // footer
    echo '</channel>'."\n";
    echo '</rss>'."\n";
  }

  /**
   * output Atom feeds (Atom Syndication Format 1.0)
   *
   * @access private
   */
  function _renderATOM() {
    // header
    header( 'Content-type: application/atom+xml' );
    echo '<?xml version="1.0" encoding="UTF-8" ?>'."\n";
    echo '<feed';
    echo ' xmlns="http://www.w3.org/2005/Atom"';
    echo ' xml:lang="'.$this->site_language.'"';
    echo ' xmlns:dc="http://purl.org/dc/elements/1.1/"';
    echo '>'."\n";
    echo '<id>'.$this->feed_url.'</id>'."\n";
    echo '<title>'.$this->site_title.'</title>'."\n";
    echo '<subtitle type="html">'.$this->site_description.'</subtitle>'."\n";
    echo '<updated>'.gmdate( XOONIPS_FEED_DATE_ATOM, time() ).'</updated>'."\n";
    echo '<link rel="alternate" type="text/html" hreflang="'.$this->site_language.'" href="'.$this->site_url.'"/>'."\n";
    echo '<link rel="self" type="application/atom+xml" href="'.$this->feed_url.'"/>'."\n";
    echo '<author>'."\n";
    echo '<name>'.$this->site_author.'</name>'."\n";
    echo '</author>'."\n";
    // entries
    $domain = 'localhost';
    $subdir = '';
    if ( preg_match( '/^(http|https):\/\/([^\/]+)(.*)?$/', XOOPS_URL, $matches ) ) {
      $domain = $matches[2];
      $subdir = $matches[3];
    }
    $date_count = array(); 
    foreach ( $this->items as $item ) {
      $date = gmdate( XOONIPS_FEED_DATE_ATOM, $item['timestamp'] );
      // generate uniq id
      $id_date = gmdate( 'Y-m-d', $item['timestamp'] );
      $id_path = $subdir.str_replace( XOOPS_URL, '', $item['link'] );
      $date_count[$date] = isset( $date_count[$date] ) ? $date_count[$date] + 1 : 0;
      $date = sprintf( '%s.%02dZ', substr( $date, 0, strlen( $date ) - 1 ), $date_count[$date] );
      echo '<entry>'."\n";
      echo '<id>tag:'.$domain.','.$id_date.':'.$id_path.','.$item['timestamp'].'</id>'."\n";
      echo '<title>'.$item['title'].'</title>'."\n";
      echo '<link rel="alternate" type="text/html" href="'.$item['link'].'"/>'."\n";
      echo '<updated>'.$date.'</updated>'."\n";
      if ( $item['description'] == '' ) {
        $item['description'] = '(empty)';
      }
      echo '<summary>'.$item['description'].'</summary>'."\n";
      echo '<dc:subject>'.$item['category'].'</dc:subject>'."\n";
      echo '</entry>'."\n";
    }
    // footer
    echo '</feed>'."\n";
  }
}

?>
