<?php
// $Revision: 1.1.2.6 $
//  ------------------------------------------------------------------------ //
//  XooNIps - Neuroinformatics Base Platform System                          //
//  Copyright (C) 2005-2007 RIKEN, Japan All rights reserved.                //
//  http://xoonips.sourceforge.jp/                                           //
//  ------------------------------------------------------------------------ //
//  This program is free software; you can redistribute it and/or modify     //
//  it under the terms of the GNU General Public License as published by     //
//  the Free Software Foundation; either version 2 of the License, or        //
//  (at your option) any later version.                                      //
//                                                                           //
//  You may not change or alter any portion of this comment or credits       //
//  of supporting developers from this source code or any supporting         //
//  source code which is considered copyrighted (c) material of the          //
//  original comment or credit authors.                                      //
//                                                                           //
//  This program is distributed in the hope that it will be useful,          //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of           //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            //
//  GNU General Public License for more details.                             //
//                                                                           //
//  You should have received a copy of the GNU General Public License        //
//  along with this program; if not, write to the Free Software              //
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA //
//  ------------------------------------------------------------------------ //

if ( ! defined( 'XOOPS_ROOT_PATH' ) ) exit();

/**
 *
 * @brief data object of file
 *
 * @li getVar('file_id') :
 * @li getVar('item_id') :
 * @li getVar('original_file_name') :
 * @li getVar('mime_type') :
 * @li getVar('file_name') :
 * @li getVar('file_size') :
 * @li getVar('thumbnail_file') :
 * @li getVar('caption') :
 * @li getVar('sess_id') :
 * @li getVar('file_type_id') :
 * @li getVar('search_module_name') :
 * @li getVar('search_module_version') :
 * @li getVar('header') :
 * @li getVar('timestamp') :
 * @li getVar('is_deleted') :
 * @li getVar('download_count') :
 *
 */
class XooNIpsFile extends XooNIpsTableObject
{

    /**
     * file path string of this file
     *
     */
    var $filepath = null;
    function XooNIpsFile() 
    {
        parent::XooNIpsTableObject();
        $this->initVar('file_id', XOBJ_DTYPE_INT, 0, false);
        $this->initVar('item_id', XOBJ_DTYPE_INT, null, false);
        $this->initVar('original_file_name', XOBJ_DTYPE_TXTBOX, null, true, 255);
        $this->initVar('mime_type', XOBJ_DTYPE_TXTBOX, null, false, 255);
        $this->initVar('file_name', XOBJ_DTYPE_TXTBOX, null, false, 10);
        $this->initVar('file_size', XOBJ_DTYPE_INT, null, false);
        $this->initVar('thumbnail_file', XOBJ_DTYPE_OTHER, null, false, 65535);
        $this->initVar('caption', XOBJ_DTYPE_TXTBOX, null, false, 255);
        $this->initVar('sess_id', XOBJ_DTYPE_TXTBOX, null, false, 32);
        $this->initVar('file_type_id', XOBJ_DTYPE_INT, null, true);
        $this->initVar('search_module_name', XOBJ_DTYPE_TXTBOX, null, false, 255);
        $this->initVar('search_module_version', XOBJ_DTYPE_OTHER, null, false);
        $this->initVar('header', XOBJ_DTYPE_TXTBOX, null, false, 32);
        $this->initVar('timestamp', XOBJ_DTYPE_TXTBOX, null, false, 14);
        $this->initVar('is_deleted', XOBJ_DTYPE_INT, 0, false);
        $this->initVar('download_count', XOBJ_DTYPE_INT, 0, false);
    }

    /**
     * @brief check that required fields are filled
     *
     * @param[in] $file file object to be checked
     * @param[out] $missing array of string of missing field name
     * @retval ture filled
     * @retval false not filled
     */
    /*
    function isFilledRequired(&$missing)
    {
    $fields = array( 'item_id', 'original_file_name', 'file_type_id');
    $missing = array();
    foreach( $fields as $field ){
    $var = $this -> getVar( $field );
    if( is_array( $var ) && 0 == count( $var )
    || empty( $var ) ){
    $missing[] = $field;
    }
    }
    
    return 0 == count( $missing );
    }
    */
    function setFilepath($path) 
    {
        $this->filepath = $path;
    }
    function getFilepath() 
    {
        return $this->filepath;
    }
}

/**
 * @brief data object of file
 */
class XooNIpsFileHandler extends XooNIpsTableObjectHandler
{
    function XooNIpsFileHandler(&$db) 
    {
        parent::XooNIpsTableObjectHandler($db);
        $this->__initHandler('XooNIpsFile', 'xoonips_file', 'file_id');
    }

    /**
     * @todo define upload_dir key name
     */
    function createFilepath($file) 
    {
        // copy file to $upload_dir
        $preference_handler = &xoonips_getormhandler('xoonips', 'config');
        $pref = &$preference_handler->get('upload_dir');
        $upload_dir = $pref->get('value');
        if (substr($upload_dir, -1) == "/") {
            $dstfilepath = $upload_dir . $file->get('file_id');
        } else {
            $dstfilepath = $upload_dir . "/" . $file->get('file_id');
        }
        return $dstfilepath;
    }

    /** 
     * @brief insert a metadata of file and move a file to sysytem's upload_dir
     *
     * @param XooNIpsFile $file file to be inserted
     * @return boolean false if failure
     */
    function insert(&$file) 
    {
        if (!$file->isDirty()) return true; // nothing to do
        if (parent::insert($file)) {
            //
            // no need to move file if update
            if (!$file->isNew()) return true;
            //
            // move file in the only case of not deleted file
            if (!$file->get('is_deleted')) return $this->moveFile($file);
            else return true;
        }
        return false;
    }

    /** 
     * @brief delete a metadata of file and remove a file
     *
     * @param XooNIpsFile $file file to delete
     * @return boolean false if failure
     */
    function delete(&$file) 
    {
        if (parent::delete($file)) {
            return true;
        }
        return false;
    }

    /** 
     * @brief get a metadata of file. returned object has a file path of this file.
     *
     * @param XooNIpsFile $file file to be inserted
     * @return boolean false if failure
     */
    function &get($id) 
    {
        $file = parent::get($id);
        if ($file) {
            $file->setFilepath($this->createFilepath($file));
        }
        return $file;
    }

    /**
     * gets File objects join xoonips_file_type.
     *
     * @access public
     * @param object $criteria
     * @param bool $id_as_key
     * @param string $fieldlist fieldlist for distinct select
     * @param bool $distinct
     * @param XoopsJoinCriteria $joindef join criteria object
     * @return array objects
     */
    function &getObjects($criteria = null, $id_as_key = false, $fieldlist = '', $distinct = false, $joindef = null) 
    {
        $table = $this->db->prefix('xoonips_file_type');
        if (is_null($joindef) || !(is_subclass_of($joindef, 'xoopsjoincriteria') || strtolower(get_class($joindef)) == 'xoopsjoincriteria')) {
            $joindef = &new XoopsJoinCriteria($table, 'file_type_id', 'file_type_id');
        } else {
            $joindef->cascade(new XoopsJoinCriteria($table, 'file_type_id', 'file_type_id'), $this->db->prefix('xoonips_file'));
        }
        $table = $this->db->prefix('xoonips_file');
        $fieldlist = "$table.file_id, $table.item_id, $table.original_file_name, $table.mime_type, $table.file_name, $table.file_size, $table.thumbnail_file, $table.caption, $table.sess_id, $table.file_type_id, $table.search_module_name, $table.search_module_version, $table.header, $table.timestamp, $table.is_deleted, $table.download_count";
        $files = parent::getObjects($criteria, $id_as_key, $fieldlist, $distinct, $joindef);
        if ($files) {
            for ($i = 0; $i < count($files); $i++) {
                $files[$i]->setFilepath($this->createFilepath($files[$i]));
            }
        }
        return $files;
    }

    /**
     * @brief move file
     *
     * $file must have file id.
     *
     * @todo define upload_dir key name
     * @param XooNIpsFile $file XooNIpsFile object
     * @return boolean false if fault
     */
    function moveFile($file) 
    {
        $file_id = $file->get('file_id');
        if (empty($file_id)) return false;
        if (rename($file->getFilepath() , $this->createFilepath($file))) {
            $file->setFilepath($this->createFilepath($file));
            return true;
        }
        trigger_error("can't move file: file_id=" . $file->get('file_id'));
        return false;
    }

    /**
     * @brief delete file
     *
     * $file must have file id.
     *
     * @param XooNIpsFile $file XooNIpsFile object
     * @return boolean false if fault
     */
    function deleteFile($file) 
    {
        $file_id = $file->get('file_id');
        if (empty($file_id)) return false;
        if (unlink($this->createFilepath($file))) {
            return true;
        }
        return false;
    }
}
?>
