<?php
// $Revision: 1.1.2.9 $
//  ------------------------------------------------------------------------ //
//  XooNIps - Neuroinformatics Base Platform System                          //
//  Copyright (C) 2005-2007 RIKEN, Japan All rights reserved.                //
//  http://xoonips.sourceforge.jp/                                           //
//  ------------------------------------------------------------------------ //
//  This program is free software; you can redistribute it and/or modify     //
//  it under the terms of the GNU General Public License as published by     //
//  the Free Software Foundation; either version 2 of the License, or        //
//  (at your option) any later version.                                      //
//                                                                           //
//  You may not change or alter any portion of this comment or credits       //
//  of supporting developers from this source code or any supporting         //
//  source code which is considered copyrighted (c) material of the          //
//  original comment or credit authors.                                      //
//                                                                           //
//  This program is distributed in the hope that it will be useful,          //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of           //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            //
//  GNU General Public License for more details.                             //
//                                                                           //
//  You should have received a copy of the GNU General Public License        //
//  along with this program; if not, write to the Free Software              //
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA //
//  ------------------------------------------------------------------------ //

if ( ! defined( 'XOOPS_ROOT_PATH' ) ) exit();

/**
 * @brief data object of index
 *
 * @li getVar('index_id') :
 * @li getVar('parent_index_id') :
 * @li getVar('uid') :
 * @li getVar('gid') :
 * @li getVar('open_level') :
 * @li getVar('sort_number') :
 *
 */
class XooNIpsIndex extends XooNIpsTableObject
{
    function XooNIpsIndex() 
    {
        parent::XooNIpsTableObject();
        $this->initVar('index_id', XOBJ_DTYPE_INT, null, true);
        $this->initVar('parent_index_id', XOBJ_DTYPE_INT, null, true);
        $this->initVar('uid', XOBJ_DTYPE_INT, null, false);
        $this->initVar('gid', XOBJ_DTYPE_INT, null, false);
        $this->initVar('open_level', XOBJ_DTYPE_INT, null, true);
        $this->initVar('sort_number', XOBJ_DTYPE_INT, null, true);
    }
    function cleanVars() 
    {
        $result = true;
        $open_level = $this->get('open_level');
        if ($open_level == OL_PUBLIC) {
            if (isset($this->vars['uid']['value'])) {
                $this->setError('cannot specify uid if open_level is OL_PUBLIC');
                $result = false;
            }
            if (isset($this->vars['gid']['value'])) {
                $this->setError('cannot specify gid if open_level is OL_PUBLIC');
                $result = false;
            }
        } else if ($open_level == OL_GROUP_ONLY) {
            if (isset($this->vars['uid']['value'])) {
                $this->setError('cannot specify uid if open_level is OL_GROUP_ONLY');
                $result = false;
            }
            $this->vars['gid']['required'] = true;
        } else if ($open_level == OL_PRIVATE) {
            $this->vars['uid']['required'] = true;
            if (isset($this->vars['gid']['value'])) {
                $this->setError('cannot specify gid if open_level is OL_PRIVATE');
                $result = false;
            }
        } else {
            $this->setError("unknown open_level($open_level)");
            $result = false;
        }
        return $result&parent::cleanVars();
    }
}

/**
 * @brief handler object of title
 *
 *
 */
class XooNIpsIndexHandler extends XooNIpsTableObjectHandler
{
    function XooNIpsIndexHandler(&$db) 
    {
        parent::XooNIpsTableObjectHandler($db);
        // TODO : change database structure. 
        // STUPID !!
        // primary key is auto_increment field but this field must 
        // synchronize with item_id of xoonips_item_basic field.
        // the seriaus problem will occurred when if this table of 
        // auto_increment seq_id conflict with already existing item_id.
        // in here, the not autoincrement flag set for temporary solution.
        $this->__initHandler('XooNIpsIndex', 'xoonips_index', 'index_id', false);
    }

    /**
     *
     * return true if permitted to this index
     *
     * @param id id of index
     * @param uid uid who access to this index
     * @param operation read|write|delete|admin (write is one of: rename, move, change sort order. admin is one of: certify item, uncertify item)
     * @return true if permitted
     *
     */
    function getPerm($id, $uid, $operation) 
    {
        if (!in_array($operation, array(
            'read',
            'write',
            'delete',
            'admin'
        ))) {
            return false; // bad operation.
            
        }
        $index = $this->get($id);
        if (false == $index){
            return false;
        }
        if ($index->get('index_id') == IID_ROOT ) {
            return false;
        }
        if ($index->get('parent_index_id') == IID_ROOT && ($operation == 'delete' || $operation == 'write')) {
            return false;
        }
        switch ($index->get('open_level')) {
            case OL_PUBLIC:
                if ($operation == 'read') {
                    return true;
                } else if ($operation == 'write' || $operation == 'delete' || $operation == 'admin') {
                    $member_handler = xoonips_gethandler('member');
                    return $member_handler->isModerator($uid);
                }
                break;

            case OL_GROUP_ONLY:
                $group_handler = xoonips_getormhandler('xoonips', 'groups');
                $criteria = new CriteriaCompo(new Criteria($this->db->prefix('xoonips_groups') . '.gid', $index->get('gid')));
                $criteria->add(new Criteria('uid', $uid));
                if ($operation == 'read') {;
                } else if ($operation == 'write' || $operation == 'delete' || $operation == 'admin') {
                    $criteria->add(new Criteria('is_admin', 1));
                } else {
                    break;
                }
                $join = new XoopsJoinCriteria($this->db->prefix('xoonips_groups_users_link') , 'gid', 'gid', 'left', 'tgul');
                $groups = $group_handler->getObjects($criteria, false, "", false, $join);
                return ($groups && count($groups) == 1);
            case OL_PRIVATE:
                return ($index->get('uid') == $uid);
            }
            return false; // must not happen.
            
        }

    /**
     *
     * create user root index
     *
     * @param uid user id
     * @return int created index id, false if failure
     * @access public
     */
    function createUserRootIndex( $uid ) {
        // check existing user index
        $criteria = new CriteriaCompo( new Criteria( 'uid', $uid ) );
        $criteria->add( new Criteria( 'parent_index_id', 1 ) );
        if ( $this->getCount( $criteria ) != 0 ) {
             return false; // already exists
        }

        // get account id (uname)
        $user_handler =& xoonips_getormhandler( 'xoonips', 'xoops_users' );
        $criteria = new Criteria( 'uid', $uid );
        $user_objs =& $user_handler->getObjects( $criteria );
        if ( count( $user_objs ) != 1 ) {
             return false; // xoops user not found
        }
        $user_obj =& $user_objs[0];
        $uname = $user_obj->getVar( 'uname', 'n' );

        // transaction
        require_once XOOPS_ROOT_PATH.'/modules/xoonips/class/base/transaction.class.php';
        $transaction =& XooNIpsTransaction::getInstance();
        $transaction->start();

        // create item basic
        $ib_handler =& xoonips_getormhandler( 'xoonips', 'item_basic' );
        $ib_obj =& $ib_handler->create();
        $ib_obj->setVar( 'item_type_id', ITID_INDEX, true ); // not gpc
        $ib_obj->setVar( 'uid', $uid, true ); // not gpc
        if ( ! $ib_handler->insert( $ib_obj ) ) {
            $transaction->rollback();
            return false;
        }

        // create item title
        $it_handler =& xoonips_getormhandler( 'xoonips', 'title' );
        $item_id = $ib_obj->getVar( 'item_id', 'n' );
        $it_obj =& $it_handler->create();
        $it_obj->setVar( 'item_id', $item_id, true ); // not gpc
        $it_obj->setVar( 'title_id', 0, true ); // not gpc
        $it_obj->setVar( 'title', $uname, true ); // not gpc
        if ( ! $it_handler->insert( $it_obj ) ) {
            $transaction->rollback();
            return false;
        }

        // get sort number
        $max_int = 2147483647;
        $criteria = new CriteriaCompo( new Criteria( 'parent_index_id', 1 ) );
        $criteria->add( new Criteria( 'open_level', 3 ) );
        if ( $this->getCount( $criteria ) == 0 ) {
           // first creation
           // this value will be used for admin on install xoonips module
           $sort_number = $max_int;
        } else {
            $idx_objs =& $this->getObjects( $criteria, false, 'MIN(sort_number) AS min_value' );
            $idx_obj =& $idx_objs[0];
            $sort_number = intval( $idx_obj->getExtraVar( 'min_value' ) ) - 1;
            unset( $idx_objs );
            unset( $idx_obj );
        }
        // create index
        $idx_obj =& $this->create();
        $idx_obj->setVar( 'index_id', $item_id, true ); // not gpc
        $idx_obj->setVar( 'parent_index_id', IID_ROOT, true ); // not gpc
        $idx_obj->setVar( 'uid', $uid, true ); // not gpc
        $idx_obj->setVar( 'open_level', OL_PRIVATE, true ); // not gpc
        $idx_obj->setVar( 'sort_number', $sort_number, true ); // not gpc
        if ( ! $this->insert( $idx_obj ) ) {
            $transaction->rollback();
            return false;
        }
        $transaction->commit();
        return $idx_obj->getVar( 'index_id', 'n' );
    }
}

?>
