<?php
// $Revision: 1.1.2.8 $
//  ------------------------------------------------------------------------ //
//  XooNIps - Neuroinformatics Base Platform System                          //
//  Copyright (C) 2005-2007 RIKEN, Japan All rights reserved.                //
//  http://xoonips.sourceforge.jp/                                           //
//  ------------------------------------------------------------------------ //
//  This program is free software; you can redistribute it and/or modify     //
//  it under the terms of the GNU General Public License as published by     //
//  the Free Software Foundation; either version 2 of the License, or        //
//  (at your option) any later version.                                      //
//                                                                           //
//  You may not change or alter any portion of this comment or credits       //
//  of supporting developers from this source code or any supporting         //
//  source code which is considered copyrighted (c) material of the          //
//  original comment or credit authors.                                      //
//                                                                           //
//  This program is distributed in the hope that it will be useful,          //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of           //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            //
//  GNU General Public License for more details.                             //
//                                                                           //
//  You should have received a copy of the GNU General Public License        //
//  along with this program; if not, write to the Free Software              //
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA //
//  ------------------------------------------------------------------------ //

if ( ! defined( 'XOOPS_ROOT_PATH' ) ) exit();

/**
 * class of XooNIps Session
 * @li getVar( 'sess_id' ) : session ID
 * @li getVar( 'updated' ) : last update time. time_t
 * @li getVar( 'uid' ) : {@link XooNIpsUser} ID
 * @li getVar( 'su_uid' ) : target uid in su mode. otherwise null.
 *
 */
class XooNIpsSession extends XooNIpsTableObject
{
    function XooNIpsSession() 
    {
        parent::XooNIpsTableObject();
        $this->initVar('sess_id', XOBJ_DTYPE_TXTBOX, null, true, 32);
        $this->initVar('updated', XOBJ_DTYPE_INT, null, true, null);
        $this->initVar('uid', XOBJ_DTYPE_INT, null, true, null);
        $this->initVar('su_uid', XOBJ_DTYPE_INT, null, false, null);
    }
}

/**
 *
 * XooNIps Session Handler class
 *
 */
class XooNIpsSessionHandler extends XooNIpsTableObjectHandler
{
    function XooNIpsSessionHandler(&$db) 
    {
        parent::XooNIpsTableObjectHandler($db);
        $this->__initHandler('XooNIpsSession', 'xoonips_session', 'sess_id', false, true);
    }

    /**
     * get objects
     *
     * @access public
     * @param object $criteria
     * @param bool $id_as_key
     * @param string $fieldlist fieldlist for distinct select
     * @param bool $distinct
     * @param XoopsJoinCriteria $joindef join criteria object
     * @return array objects
     */
    function &getObjects($criteria = null, $id_as_key = false, $fieldlist = '', $distinct = false, $joindef = null) 
    {
        if ($fieldlist == '') $fieldlist = '*';
        $fieldlist.= ", unix_timestamp(" . $this->db->prefix($this->__table_name) . ".updated) as updated ";
        return parent::getObjects($criteria, $id_as_key, $fieldlist, $distinct, $joindef);
    }

    /**
     * helper function for sql string creation. change last_updated to updated
     *
     * @access private
     * @param object $obj
     * @param array $vars array of variables
     * @return array quoted strings
     */
    function &_makeVarsArray4SQL(&$obj, &$vars) 
    {
        $ret = parent::_makeVarsArray4SQL($obj, $vars);
        $ret['updated'] = "from_unixtime(" . $ret['updated'] . ")";
        return $ret;
    }

    /**
     * create xoonips session
     */
    function createSession( $uid )
    {
        $session =& $this->create();
        $session->setVar( 'sess_id', session_id(), true ); // not gpc
        $session->setVar( 'uid', $uid, true ); // not gpc
        $session->setVar( 'updated', time(), true ); // not gpc
        return $this->insert( $session );
    }

    /**
     * GC expired sessions
     */
    function gcSession()
    {
        $session_expire = $GLOBALS['xoopsConfig']['session_expire'];
        $expire_time = time() + ( 60 * $session_expire );
        $eventlog_handler =& xoonips_getormhandler( 'xoonips', 'event_log' );

        $join = new XooNIpsJoinCriteria( 'session', 'sess_id', 'sess_id', 'LEFT', 'ts');
        $criteria = new Criteria( 'ISNULL(ts.sess_id)', 1 );
        // set fieldlist for same field name 'sess_id'
        $fieldlist = $GLOBALS['xoopsDB']->prefix('xoonips_session').'.*';
        $sessions =& $this->getObjects( $criteria, false, $fieldlist, false, $join );
        foreach( $sessions as $session ) {
            $su_uid = $session->get( 'su_uid' );
            $uid = $session->get( 'uid' );
            // terminate switch user mode
            if ( $su_uid ) {
                if ( ! $eventlog_handler->recordEndSuEvent( $uid, $su_uid, $expire_time ) ) {
                    return false;
                }
            }
            // insert logout event
            if ( ! $eventlog_handler->recordLogoutEvent( $uid, $expire_time ) ) {
                return false;
            }
            // remove xoonips session
            if ( ! $this->delete( $session ) ) {
                return false;
            }
        }
        return true;
    }
}

?>
