<?php
// $Revision: 1.1.2.4 $
//  ------------------------------------------------------------------------ //
//  XooNIps - Neuroinformatics Base Platform System                          //
//  Copyright (C) 2005-2007 RIKEN, Japan All rights reserved.                //
//  http://xoonips.sourceforge.jp/                                           //
//  ------------------------------------------------------------------------ //
//  This program is free software; you can redistribute it and/or modify     //
//  it under the terms of the GNU General Public License as published by     //
//  the Free Software Foundation; either version 2 of the License, or        //
//  (at your option) any later version.                                      //
//                                                                           //
//  You may not change or alter any portion of this comment or credits       //
//  of supporting developers from this source code or any supporting         //
//  source code which is considered copyrighted (c) material of the          //
//  original comment or credit authors.                                      //
//                                                                           //
//  This program is distributed in the hope that it will be useful,          //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of           //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            //
//  GNU General Public License for more details.                             //
//                                                                           //
//  You should have received a copy of the GNU General Public License        //
//  along with this program; if not, write to the Free Software              //
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA //
//  ------------------------------------------------------------------------ //

if ( ! defined( 'XOOPS_ROOT_PATH' ) ) exit();

include dirname( dirname( __FILE__ ) ).'/class/base/tableobject.class.php';

/**
 *
 * @brief get XoopsObjectHandler of xoonips
 *
 * @param[in] $name handler name
 * @return reference of XoopsObjectHandler
 * @retval false
 */
function &xoonips_gethandler($name) 
{
    static $falseVar = false;
    static $handlers;
    //
    if (!isset($handlers[$name])) {
        $include_file = XOOPS_ROOT_PATH . "/modules/xoonips/class/xoonips{$name}.class.php";
        if (file_exists($include_file)) {
            include_once $include_file;
        } else {
            return $falseVar;
        }
        $class = 'XooNIps' . ucfirst($name) . 'Handler';
        if (class_exists($class)) {
            $handlers[$name] = new $class($GLOBALS['xoopsDB']);
        }
    }
    if (!isset($handlers[$name])) {
        trigger_error('Handler does not exist. Name: ' . $name, E_USER_ERROR);
    }
    // return result
    $falseVar = false;
    if (isset($handlers[$name])) return $handlers[$name];
    else return $falseVar;
}

/**
 *
 * @brief get handler of xoonips
 *
 * @param[in] $name handler name
 * @return XoopsTableObjectHandler
 * @retval false
 */
function &xoonips_getormhandler($module, $name) 
{
    static $falseVar = false;
    static $handlers;
    //
    if (!isset($handlers[$module . $name])) {
        $include_file = XOOPS_ROOT_PATH . "/modules/${module}/class/orm/${name}.class.php";
        if (file_exists($include_file)) {
            include_once $include_file;
        } else {
            trigger_error("can't open file: " . $include_file);
            return $falseVar;
        }
        if (strncmp('xnp', $module, 3) == 0) {
            $tok = split('xnp', $module);
            if (count($tok) != 2) return $falseVar;
            $class = 'XNP' . ucfirst($tok[1]) . str_replace(' ', '', ucwords(str_replace('_', ' ', $name))) . 'Handler';
        } else {
            $class = 'XooNIps' . str_replace(' ', '', ucwords(str_replace('_', ' ', $name))) . 'Handler';
        }
        if (class_exists($class)) {
            $handlers[$module . $name] = new $class($GLOBALS['xoopsDB']);
        }
    }
    if (!isset($handlers[$module . $name])) {
        trigger_error('Handler does not exist. Class: ' . $class, E_USER_ERROR);
    }
    // return result
    if (isset($handlers[$module . $name])) return $handlers[$module . $name];
    else return $falseVar;
}

/**
 *
 * get XooNIpsItemHandler of specified itemtype
 *
 * @param[in] $module module name
 * @param[in] $name handler name
 * @return XooNIpsItemCompoHandler
 * @retval false
 */
function &xoonips_getormcompohandler($module, $name) 
{
    static $falseVar = false;
    static $handlers;
    //
    if (!isset($handlers[$module . $name])) {
        $include_file = XOOPS_ROOT_PATH . "/modules/${module}/class/${module}_${name}.class.php";
        if (file_exists($include_file)) {
            include_once $include_file;
        } else {
            return $falseVar;
        }
        if (strncmp('xnp', $module, 3) == 0) {
            $tok = split('xnp', $module);
            if (count($tok) != 2) return $falseVar;
            $class = 'XNP' . ucfirst($tok[1]) . 'CompoHandler';
        } else {
            $class = 'XooNIps' . ucfirst($name) . 'CompoHandler';
        }
        if (class_exists($class)) {
            $handlers[$module . $name] = new $class($GLOBALS['xoopsDB']);
        }
    }
    if (!isset($handlers[$module . $name])) {
        trigger_error('Handler does not exist. Name: ' . $module . ' ' . $name, E_USER_ERROR);
    }
    // return result
    if (isset($handlers[$module . $name])) return $handlers[$module . $name];
    else return $falseVar;
}
function ISO8601toUnixTimestamp($str) 
{
    if (preg_match('/^([0-9]{4})(-?([0-9]{2})(-?([0-9]{2})(T([0-9]{2}):([0-9]{2})(:([0-9]{2}))?(Z|([-+])([0-9]{2})([0-9]{2}))?)?)?)?$/', $str, $match) == 1) {
        // $match[?]
        // $match[0]  : input($str)
        // $match[1]  : year
        // $match[2]  :
        // $match[3]  : month
        // $match[4]  :
        // $match[5]  : day of month
        // $match[6]  :
        // $match[7]  : hour
        // $match[8]  : minute
        // $match[9]  :
        // $match[10] : second
        // time difference below(regexp Z|[-+][0-9]{2}:[0-9]{2})
        // $match[11] : 'Z' or ''
        // $match[12] : +|-
        // $match[13] : hour of time difference
        // $match[14] : minute of time difference
        if (!isset($match[3])) $match[3] = '01';
        if (!isset($match[5])) $match[5] = '01';
        if (!isset($match[7])) $match[7] = '00';
        if (!isset($match[8])) $match[8] = '00';
        if (!isset($match[10]) || $match[10] == "") $match[10] = '00';
        $tm = gmmktime($match[7], $match[8], $match[10], $match[3], $match[5], $match[1]);
        if (false === $tm || -1 == $tm && version_compare(phpversion() , "5.1.0", "<")) return false; // gmmktime failed.
        // hh:mm:ss must be in 00:00:00 - 24:00:00
        if ($match[10] >= 60) return false;
        if ($match[8] >= 60) return false;
        if ($match[7] > 24 || $match[7] == 24 && ($match[8] != 0 || $match[10] != 0)) return false;
        // mm and dd must not overflow
        if (gmdate('Ymd', gmmktime(0, 0, 0, $match[3], $match[5], $match[1])) != $match[1] . $match[3] . $match[5]) return false;
        //correct a time difference to GMT
        if (isset($match[11]) && isset($match[12]) && isset($match[13]) && isset($match[14])) {
            if ($match[11] != 'Z' && $match[12] == '-') {
                $tm = $tm+($match[13]*3600+$match[14]*60);
            } else if (isset($match[12]) && $match[11] != 'Z' && $match[12] == '+') {
                $tm = $tm-($match[13]*3600+$match[14]*60);
            }
        }
    } else if (preg_match('/^([0-9]{4})(-W([0-5][0-9]))(-([1-7]))$/', $str, $match) == 1) {
        $dt = false;
        //          if (!$dt ) $dt = strptime($str,"%Y-W%W-%w");//YYYY-Www-D
        //          if (!$dt ) $dt = strptime($str,"%Y");//YYYY
        //          if (!$dt ) $dt = strptime($str,"%Y-%j");//YYYY-DDD
        //          if (!$dt ) $dt = strptime($str,"%Y-%m");//YYYY-MM
        //          if (!$dt ) $dt = strptime($str,"%Y%m");//YYYYMM
        //          if (!$dt ) $dt = strptime($str,"%Y-%m-%d");//YYYY-MM-DD
        //          if (!$dt ) $dt = strptime($str,"%Y%m%d");//YYYYMMDD
        //          if (!$dt ) $dt = strptime($str,"%Y-W%W");//YYYY-Www
        //          if (!$dt ) $dt = strptime($str,"%YW%W");//YYYYWww
        //          if (!$dt ) $dt = strptime($str,"%YW%W%w");//YYYYWwwD
        //          if (!$dt ) $dt = strptime($str,"%Y%j");//YYYYDDD
        //          var_dump($str, $dt );
        //          return gmmktime( 0, 0, 0, $dt['tm_mon'] + 1 , $dt['tm_mday'], $dt['tm_year'] + 1900 );
        // Week dates format
        $y = $match[1];
        $w = $match[3];
        $d = $match[5];
        $days = array(
            7,
            1,
            2,
            3,
            4,
            5,
            6
        ); //map 0:sunday-6:saturday -> 1:monday-7:sunday
        $dayofweek = $days[dayOfWeek($y, 1, 1) /*0:sunday-6:saturday*/
        ];
        //var_dump($match, $dayofweek);
        $t = gmmktime(0, 0, 0, 1, 1, $match[1]);
        $tm = $t+(($w-1) *7+$d-$dayofweek) *86400;
    } else if (preg_match('/^([0-9]{4})(-?([0-3][0-9]{2}))$/', $str, $match) == 1) {
        // Ordinal dates format
        $tm = gmmktime(0, 0, 0, 1, 1, $match[1]) +($match[3]-1) *86400;
    } else return false;
    return $tm;
}
function dayOfWeek($year, $month, $day) 
{
    /*0:sunday-6:saturday*/
    //echo "dayOfWeek($year, $month, $day)\n";
    if ($month == 1 || $month == 2) {
        $year--;
        $month+= 12;
    }
    return ($year+$year/4-$year/100+$year/400+2.6*$month+1.6+$day) %7;
}
?>
